@@ -11,8 +11,7 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 """mercurial interactive history viewer
 
 Its purpose is similar to the hgk tool of mercurial, and it has been
@@ -20,10 +19,3 @@ written with efficiency in mind when dealing with big repositories
 (it can happily be used to browse Linux kernel source code
 repository).
 """
-
-# monkey patch to support older hg versions
-from mercurial import changelog, filelog
-if not hasattr(changelog.changelog, '__len__'):
-    changelog.changelog.__len__ = changelog.changelog.count
-if not hasattr(filelog.filelog, '__len__'):
-    filelog.filelog.__len__ = filelog.filelog.count
