@@ -12,8 +12,7 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 # pylint: disable-msg=C0103
 
@@ -21,8 +20,10 @@
 Module for managing configuration parameters of hgview using Hg's
 configuration system
 """
+from functools import partial
 import os
 import re
+import shlex
 
 def cached(meth):
     """
@@ -47,6 +48,22 @@ class HgConfig(object):
         self.section = section
         self._cache = {}
 
+    @cached
+    def getFancyReplace(self):
+        r"""
+        fancyreplace: ``"patt" "repl"`` used to modify description by replacing
+            ``patt`` by ``repl`` using regular expression (``re.sub`` for instance).
+            Ex: "#(\d+)":"`#\\1 <http://www.logilab.org/ticket/\\1>`_"
+        """
+        data = self.ui.config(self.section, 'fancyreplace', None)
+        if data is None:
+            return data
+        data = shlex.split(data)
+        assert len(data) == 2
+        patt, repl = data
+        return partial(re.compile(patt).sub, repl)
+
+
     @cached
     def getFont(self):
         """
@@ -240,6 +257,20 @@ class HgConfig(object):
         """
         return self.ui.config(self.section, 'mqhidetags', default)
 
+    @cached
+    def getShowHidden(self, default=False):
+        """
+        showhidden: show hidden changeset at startup
+        """
+        return bool(self.ui.config(self.section, 'showhidden', default))
+
+    @cached
+    def getInterface(self, default=None):
+        """
+        interface: which GUI interface to use (among "qt", "raw" and "curses")
+        """
+        return self.ui.config(self.section, 'interface', default)
+
 
 
 _HgConfig = HgConfig
