@@ -11,43 +11,59 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 """helper functions and classes to ease hg revision graph building
 
 Based on graphlog's algorithm, with insipration stolen to TortoiseHg
 revision grapher.
 """
 
+import os
 from cStringIO import StringIO
 import difflib
+from itertools import chain
+from time import strftime, localtime
 
 from mercurial.node import nullrev
-from mercurial import patch, util, match
+from mercurial import patch, util, match, error, hg
+
+import hgviewlib.hgpatches # force apply patches to mercurial
+from hgviewlib.hgpatches import mqsupport
 
-import hgviewlib # force apply monkeypatches
 from hgviewlib.util import tounicode, isbfile
+from hgviewlib.config import HgConfig
+
+DATE_FMT = '%F %R'
 
 def diff(repo, ctx1, ctx2=None, files=None):
     """
-    Compute the diff of files between 2 changectx
+    Compute the diff of ``files`` between the 2 contexts ``ctx1`` and ``ctx2``.
+
+    :Note: context may be a changectx or a filectx.
+
+    * If ``ctx2`` is None, the parent of ``ctx1`` is used. If ``ctx1`` is a
+      file ctx, the parent is the first ancestor that contains modification
+      on the given file
+    * If ``files`` is None, return the diff for all files.
+
     """
+    if not getattr(ctx1, 'applied', True): #no diff vs ctx2 on unapplied patch
+        return ''.join(chain(ctx1.filectx(fname).data() for fname in files))
     if ctx2 is None:
-        ctx2 = ctx1.parents()[0]
+        ctx2 = ctx1.p1()
     if files is None:
-        m = match.always(repo.root, repo.getcwd())
+        matchfn = match.always(repo.root, repo.getcwd())
     else:
-        m = match.exact(repo.root, repo.getcwd(), files)
+        matchfn = match.exact(repo.root, repo.getcwd(), files)
     # try/except for the sake of hg compatibility (API changes between
     # 1.0 and 1.1)
     try:
         out = StringIO()
-        patch.diff(repo, ctx2.node(), ctx1.node(), match=m, fp=out)
+        patch.diff(repo, ctx2.node(), ctx1.node(), match=matchfn, fp=out)
         diffdata = out.getvalue()
     except:
         diffdata = '\n'.join(patch.diff(repo, ctx2.node(), ctx1.node(),
-                                        match=m))
+                                        match=matchfn))
     # XXX how to deal diff encodings?
     try:
         diffdata = unicode(diffdata, "utf-8")
@@ -73,6 +89,28 @@ def __get_parents(repo, rev, branch=None):
     return [x for x in repo.changelog.parentrevs(rev) \
             if (x != nullrev and repo.changectx(rev).branch() == branch)]
 
+def getlog(model, ctx, gnode):
+    if ctx.rev() is not None:
+        msg = tounicode(ctx.description())
+        if msg:
+            msg = msg.splitlines()[0]
+    else:
+        msg = "WORKING DIRECTORY (locally modified)"
+    return msg
+
+def gettags(model, ctx, gnode):
+    if ctx.rev() is None:
+        return ""
+    mqtags = ['qbase', 'qtip', 'qparent']
+    tags = ctx.tags()
+    if model.hide_mq_tags:
+        tags = [t for t in tags if t not in mqtags]
+    return ",".join(tags)
+
+def getdate(model, ctx, gnode):
+    if not ctx.date():
+        return ""
+    return strftime(DATE_FMT, localtime(int(ctx.date()[0])))
 
 def ismerge(ctx):
     """
@@ -83,7 +121,7 @@ def ismerge(ctx):
         return len(ctx.parents()) == 2 and ctx.parents()[1]
     return False
 
-def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False):
+def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False, show_hidden=False):
     """incremental revision grapher
 
     This generator function walks through the revision history from
@@ -98,10 +136,21 @@ def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False
         the current row and the next row
       - parent revisions of current revision
 
+    If start_rev is -1, shown unapplied mq patches (if available)
+
+    If start_rev is None, started from working directory node
+
     If follow is True, only generated the subtree from the start_rev head.
 
     If branch is set, only generated the subtree for the given named branch.
+
     """
+    if show_hidden and start_rev == None and hasattr(repo, 'mq'):
+        series = list(reversed(repo.mq.series))
+        for patchname in series:
+            if not repo.mq.isapplied(patchname):
+                yield (patchname, 0, 0, [(0,0,0)], [])
+
     if start_rev is None and repo.status() == ([],)*7:
         start_rev = len(repo.changelog)
     assert start_rev is None or start_rev >= stop_rev
@@ -109,12 +158,18 @@ def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False
     revs = []
     rev_color = {}
     nextcolor = 0
+    hiddenrevs = getattr(repo.changelog, 'hiddenrevs', ())
     while curr_rev is None or curr_rev >= stop_rev:
         # Compute revs and next_revs.
+        if (not show_hidden) and curr_rev in hiddenrevs:
+            if curr_rev is None:
+                curr_rev = len(repo.changelog)
+            else:
+                curr_rev -= 1
+            continue
         if curr_rev not in revs:
             if branch:
-                ctx = repo.changectx(curr_rev)
-                if ctx.branch() != branch:
+                if repo[curr_rev].branch() != branch:
                     if curr_rev is None:
                         curr_rev = len(repo.changelog)
                     else:
@@ -284,7 +339,6 @@ class Graph(object):
         """
         Build up to `nnodes` more nodes in our graph, or build as many
         nodes required to reach `rev`.
-
         If both rev and nnodes are set, build as many nodes as
         required to reach rev plus nnodes more.
         """
@@ -296,7 +350,7 @@ class Graph(object):
             if vnext is None:
                 continue
             nrev, xpos, color, lines, parents = vnext[:5]
-            if nrev >= self.maxlog:
+            if isinstance(nrev, int) and nrev >= self.maxlog:
                 continue
             gnode = GraphNode(nrev, xpos, color, lines, parents,
                               extra=vnext[5:])
@@ -358,15 +412,20 @@ class Graph(object):
             return self.nodes.index(self.nodesdict[rev])
         return -1
 
-    def fileflags(self, filename, rev):
+    def fileflags(self, filename, rev, _cache={}):
         """
         Return a couple of flags ('=', '+', '-' or '?') depending on the nature
         of the diff for filename between rev and its parents.
         """
-        ctx = self.repo.changectx(rev)
+        if rev not in _cache:
+            ctx = self.repo.changectx(rev)
+            _cache.clear()
+            _cache[rev] = (ctx,
+                           [self.repo.status(p.node(), ctx.node())[:5]
+                           for p in ctx.parents()])
+        ctx, allchanges = _cache[rev]
         flags = []
-        for p in ctx.parents():
-            changes = self.repo.status(p.node(), ctx.node())[:5]
+        for changes in allchanges:
             # changes = modified, added, removed, deleted, unknown
             for flag, lst in zip(["=", "+", "-", "-", "?"], changes):
                 if filename in lst:
@@ -383,24 +442,30 @@ class Graph(object):
 
     def fileflag(self, filename, rev):
         """
-        Return a flag (see fileflags) between rev and its first parent
+        Return a flag (see fileflags) between rev and its first parent (may be
+        long)
         """
         return self.fileflags(filename, rev)[0]
 
     def filename(self, rev):
         return self.nodesdict[rev].extra[0]
 
-    def filedata(self, filename, rev, mode='diff'):
+    def filedata(self, filename, rev, mode='diff', flag=None):
         """XXX written under dubious encoding assumptions
+
+        The modification flag is computed using *fileflag* if ``flag`` is None.
         """
         # XXX This really begins to be a dirty mess...
         data = ""
-        flag = self.fileflag(filename, rev)
+        if flag is None:
+            flag = self.fileflag(filename, rev)
         ctx = self.repo.changectx(rev)
+        filesize = 0
         try:
             fctx = ctx.filectx(filename)
+            filesize = fctx.size() # compute size here to lookup data securely
         except LookupError:
-            fctx = None # may happen for renamed files?
+            fctx = None # may happen for renamed files or mq patch ?
 
         if isbfile(filename):
             data = "[bfile]\n"
@@ -411,7 +476,7 @@ class Graph(object):
         if flag not in ('-', '?'):
             if fctx is None:# or fctx.node() is None:
                 return '', None
-            if fctx.size() > self.maxfilesize:
+            if filesize > self.maxfilesize:
                 data = "file too big"
                 return flag, data
             if flag == "+" or mode == 'file':
@@ -427,11 +492,9 @@ class Graph(object):
                 if isinstance(mode, int):
                     parentctx = self.repo.changectx(mode)
                 else:
-                    parent = self.fileparent(filename, rev)
-                    parentctx = self.repo.changectx(parent)
-                # return the diff but the 3 first lines
+                    parentctx = self.repo[self._fileparent(fctx)]
                 data = diff(self.repo, ctx, parentctx, files=[filename])
-                data = u'\n'.join(data.splitlines()[3:])
+                data = data.split(os.linesep, 3)[-1]
             elif flag == '':
                 data = ''
             else: # file renamed
@@ -449,20 +512,182 @@ class Graph(object):
                 data = u'\n'.join(tounicode(elt) for elt in data)
         return flag, data
 
+    def _fileparent(self, fctx):
+        try:
+            return fctx.p1().rev()
+        except IndexError: # reach bottom
+            return -1
+
     def fileparent(self, filename, rev):
+        return self._fileparent(self.repo[rev].filectx(filename))
+
+class HgRepoListWalker(object):
+    """
+    Graph object to ease hg repo revision tree drawing depending on user's
+    configurations.
+    """
+    _allcolumns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Tags',)
+    _columns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Tags',)
+    _stretchs = {'Log': 1, }
+    _getcolumns = "getChangelogColumns"
+
+    def __init__(self, repo, branch='', fromhead=None, follow=False,
+                 parent=None, *args, **kwargs):
+        """
+        repo is a hg repo instance
+        """
+        #XXX col radius
+        self._datacache = {}
+        self._hasmq = False
+        self.mqueues = []
+        self.wd_revs = []
+        self.graph = None
+        self.rowcount = 0
+        self.repo = repo
+        self.show_hidden = False
+        super(HgRepoListWalker, self).__init__()
+        self.load_config()
+        self.setRepo(repo, branch=branch, fromhead=fromhead, follow=follow)
+
+    def setRepo(self, repo=None, branch='', fromhead=None, follow=False):
+        if repo is None:
+            repo = hg.repository(self.repo.ui, self.repo.root)
+        self._hasmq = hasattr(self.repo, "mq")
+        if not getattr(repo, '__hgview__', False) and self._hasmq:
+            mqsupport.reposetup(repo.ui, repo)
+        oldrepo = self.repo
+        self.repo = repo
+        if oldrepo.root != repo.root:
+            self.load_config()
+        self._datacache = {}
+        try:
+            wdctxs = self.repo.changectx(None).parents()
+        except error.Abort:
+            # might occur if reloading during a mq operation (or
+            # whatever operation playing with hg history)
+            return
+        if self._hasmq:
+            self.mqueues = self.repo.mq.series[:]
+        self.wd_revs = [ctx.rev() for ctx in wdctxs]
+        self.wd_status = [self.repo.status(ctx.node(), None)[:4] for ctx in wdctxs]
+        self._user_colors = {}
+        self._branch_colors = {}
+        # precompute named branch color for stable value.
+        for branch_name in chain(['default', 'stable'], sorted(repo.branchtags().keys())):
+            self.namedbranch_color(branch_name)
+        grapher = revision_grapher(self.repo, start_rev=fromhead,
+                                   follow=follow, branch=branch,
+                                   show_hidden=self.show_hidden)
+        self.graph = Graph(self.repo, grapher, self.max_file_size)
+        self.rowcount = 0
+        self.heads = [self.repo.changectx(x).rev() for x in self.repo.heads()]
+        self.ensureBuilt(row=self.fill_step)
+
+    def ensureBuilt(self, rev=None, row=None):
+        """
+        Make sure rev data is available (graph element created).
+
+        """
+        if self.graph.isfilled():
+            return
+        required = 0
+        buildrev = rev
+        n = len(self.graph)
         if rev is not None:
-            node = self.repo.changelog.node(rev)
-        else:
-            node = self.repo.changectx(rev).node()
-        for parent in self.nodesdict[rev].parents:
-            pnode = self.repo.changelog.node(parent)
-            changes = self.repo.status(pnode, node)[:5]
-            allchanges = []
-            [allchanges.extend(e) for e in changes]
-            if filename in allchanges:
-                return parent
+            if n and self.graph[-1].rev <= rev:
+                buildrev = None
+            else:
+                required = self.fill_step / 2
+        elif row is not None and row > (n - self.fill_step / 2):
+            required = row - n + self.fill_step
+        if required or buildrev:
+            self.graph.build_nodes(nnodes=required, rev=buildrev)
+            self.updateRowCount()
+        elif row and row > self.rowcount:
+            # asked row was already built, but views where not aware of this
+            self.updateRowCount()
+        elif rev is not None and rev <= self.graph[self.rowcount].rev:
+            # asked rev was already built, but views where not aware of this
+            self.updateRowCount()
+
+    def updateRowCount(self):
+        self.rowcount = None
+        #raise NotImplementedError
+
+    def rowCount(self, parent=None):
+        return self.rowcount
+
+    def columnCount(self, parent=None):
+        return len(self._columns)
+
+    def load_config(self):
+        cfg = HgConfig(self.repo.ui)
+        self._users, self._aliases = cfg.getUsers()
+        self.dot_radius = cfg.getDotRadius(default=8)
+        self.rowheight = cfg.getRowHeight()
+        self.fill_step = cfg.getFillingStep()
+        self.max_file_size = cfg.getMaxFileSize()
+        self.hide_mq_tags = cfg.getMQHideTags()
+        self.show_hidden = cfg.getShowHidden()
+
+        cols = getattr(cfg, self._getcolumns)()
+        if cols is not None:
+            validcols = [col for col in cols if col in self._allcolumns]
+            if len(validcols) != len(cols):
+                wrongcols = [col for col in cols if col not in self._allcolumns]
+                #XXX
+                #print "WARNING! %s are not valid column names. Check your configuration." % ','.join(wrongcols)
+                #print "         reverting to default columns configuration"
+            elif 'Log' not in validcols or 'ID' not in validcols:
+                pass
+                #print "WARNING! 'Log' and 'ID' are mandatory. Check your configuration."
+                #print "         reverting to default columns configuration"
+            else:
+                self._columns = tuple(validcols)
+
+    @staticmethod
+    def get_color(n, ignore=()):
+        return []
+
+    def user_color(self, user):
+        if user not in self._user_colors:
+            self._user_colors[user] = self.get_color(len(self._user_colors),
+                                                self._user_colors.values())
+        return self._user_colors[user]
+
+    def user_name(self, user):
+        return self._aliases.get(user, user)
+
+    def namedbranch_color(self, branch):
+        if branch not in self._branch_colors:
+            self._branch_colors[branch] = self.get_color(len(self._branch_colors))
+        return self._branch_colors[branch]
+
+    def col2x(self, col):
+        return (1.2*self.dot_radius + 0) * col + self.dot_radius/2 + 3
+
+    def rowFromRev(self, rev):
+        row = self.graph.index(rev)
+        if row == -1:
+            row = None
+        return row
+
+    def indexFromRev(self, rev):
+        self.ensureBuilt(rev=rev)
+        row = self.rowFromRev(rev)
+        if row is not None:
+            return self.index(row, 0)
         return None
 
+    def clear(self):
+        """empty the list"""
+        self.graph = None
+        self._datacache = {}
+        self.notify_data_changed()
+
+    def notify_data_changed(self):
+        pass
+
 if __name__ == "__main__":
     # pylint: disable-msg=C0103
     import sys
