@@ -12,8 +12,7 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #
 # make sure the Qt rc files are converted into python modules, then load them
@@ -21,7 +20,7 @@
 import os
 import os.path as osp
 import sys
-import mx.DateTime as dt
+import datetime as dt
 
 def should_rebuild(srcfile, pyfile):
     return not osp.isfile(pyfile) or osp.isfile(srcfile) and \
@@ -50,7 +49,7 @@ import hgqv_rc
 
 _icons = {}
 def _load_icons():
-    t = dt.today()
+    t = dt.date.today()
     x = t.month == 12 and t.day in (24,25)
     d = QtCore.QDir(':/icons')
     for icn in d.entryList():
