@@ -12,8 +12,7 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 Qt4 dialogs to display hg revisions of a file
 """
@@ -28,6 +27,8 @@ from mercurial import ui, hg, util
 from PyQt4 import QtGui, QtCore, Qsci
 from PyQt4.QtCore import Qt
 
+from hgviewlib.application import (FileViewer as _FileViewer,
+                                   FileDiffViewer as _FileDiffViewer)
 from hgviewlib.util import tounicode, rootpath
 
 from hgviewlib.qt4 import icon as geticon
@@ -91,7 +92,7 @@ class AbstractFileDialog(QtGui.QMainWindow, HgDialogMixin):
     def modelFilled(self):
         disconnect(self.filerevmodel, SIGNAL('filled'),
                    self.modelFilled)
-        if self._show_rev is not None:
+        if isinstance(self._show_rev, int):
             index = self.filerevmodel.indexFromRev(self._show_rev)
             self._show_rev = None
         else:
@@ -111,7 +112,7 @@ class AbstractFileDialog(QtGui.QMainWindow, HgDialogMixin):
         self.repoviewer.activateWindow()
         self.repoviewer.raise_()
 
-class FileViewer(AbstractFileDialog):
+class FileViewer(AbstractFileDialog, _FileViewer):
     """
     A dialog showing a revision graph for a file.
     """
@@ -224,7 +225,7 @@ class FileViewer(AbstractFileDialog):
         self.actionNextDiff.setEnabled(self.textView.fileMode() and self.textView.nDiffs())
 
 
-class FileDiffViewer(AbstractFileDialog):
+class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
     """
     Qt4 dialog to display diffs between different mercurial revisions of a file.
     """
