@@ -11,8 +11,7 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 Qt4 high level widgets for hg repo changelogs and filelogs
 """
@@ -106,6 +105,9 @@ class HgFileView(QtGui.QFrame):
         self.topLayout = QtGui.QVBoxLayout()
         self.filenamelabel = QtGui.QLabel()
         self.filenamelabel.setWordWrap(True)
+        self.filenamelabel.setTextInteractionFlags(
+            QtCore.Qt.TextSelectableByKeyboard|
+            QtCore.Qt.TextSelectableByMouse)
         self.execflaglabel = QtGui.QLabel()
         self.execflaglabel.setWordWrap(True)
         self.topLayout.addWidget(self.filenamelabel)
