@@ -12,8 +12,7 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 Qt4 dialogs to display hg revisions of a file
 """
@@ -27,6 +26,7 @@ from mercurial.revlog import LookupError
 from PyQt4 import QtGui, QtCore, Qsci
 from PyQt4.QtCore import Qt
 
+from hgviewlib.application import ManifestViewer as _ManifestViewer
 from hgviewlib.util import tounicode
 
 from hgviewlib.qt4 import icon as geticon
@@ -40,7 +40,7 @@ SIGNAL = QtCore.SIGNAL
 nullvariant = QtCore.QVariant()
 
 
-class ManifestViewer(QtGui.QMainWindow, HgDialogMixin):
+class ManifestViewer(QtGui.QMainWindow, HgDialogMixin, _ManifestViewer):
     """
     Qt4 dialog to display all files of a repo at a given revision
     """
