@@ -11,8 +11,7 @@
 # FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 Qt4 high level widgets for hg repo changelogs and filelogs
 """
@@ -307,11 +306,13 @@ class HgRepoView(QtGui.QTableView):
         """
         Select revision 'rev' (can be anything understood by repo.changectx())
         """
+        if isinstance(rev, basestring) and ':' in rev:
+            rev = rev.split(':')[1]
         repo = self.model().repo
         try:
             rev = repo.changectx(rev).rev()
         except RepoError:
-            self.emit(SIGNAL('showMessage'), "Can't find revision '%s'"%rev, 2000)
+            self.emit(SIGNAL('showMessage'), "Can't find revision '%s'" % rev, 2000)
         else:
             idx = self.model().indexFromRev(rev)
             if idx is not None:
@@ -362,8 +363,11 @@ class RevDisplay(QtGui.QTextBrowser):
             self.refreshDisplay()
             # TODO: emit a signal to recompute the diff
             self.emit(SIGNAL('parentRevisionSelected'), self.diffrev)
+        elif rev.isdigit():
+            self.emit(SIGNAL('revisionSelected'), rev)
         else:
-            self.emit(SIGNAL('revisionSelected'), int(rev))
+            QtGui.QDesktopServices.openUrl(qurl)
+            self.refreshDisplay()
 
     def setDiffRevision(self, rev):
         if rev != self.diffrev:
@@ -418,9 +422,10 @@ class RevDisplay(QtGui.QTextBrowser):
     def refreshDisplay(self):
         ctx = self.ctx
         rev = ctx.rev()
+        cfg = HgConfig(ctx._repo.ui)
         buf = "<table width=100%>\n"
         if self.mqpatch:
-            buf += '<tr bgcolor=%s>' % HgConfig(ctx._repo.ui).getMQFGColor()
+            buf += '<tr bgcolor=%s>' % cfg.getMQFGColor()
             buf += '<td colspan=3 width=100%><b>Patch queue:</b>&nbsp;'
             for p in self.mqseries:
                 if p in self.mqunapplied:
@@ -435,7 +440,7 @@ class RevDisplay(QtGui.QTextBrowser):
             buf += "<td><b>Working Directory</b></td>\n"
         else:
             buf += '<td><b>Revision:</b>&nbsp;'\
-                   '<span class="rev_number">%d</span>:'\
+                   '<span class="rev_number">%s</span>:'\
                    '<span class="rev_hash">%s</span></td>'\
                    '\n' % (ctx.rev(), short_hex(ctx.node()))
 
@@ -449,7 +454,7 @@ class RevDisplay(QtGui.QTextBrowser):
         parents = [p for p in ctx.parents() if p]
         for p in parents:
             if p.rev() > -1:
-                short = short_hex(p.node())
+                short = short_hex(p.node()) if getattr(p, 'applied', True) else p.node()
                 desc = format_desc(p.description(), self.descwidth)
                 p_rev = p.rev()
                 p_fmt = '<span class="rev_number">%s</span>:'\
@@ -480,10 +485,10 @@ class RevDisplay(QtGui.QTextBrowser):
 
         for p in ctx.children():
             if p.rev() > -1:
-                short = short_hex(p.node())
+                short = short_hex(p.node()) if getattr(p, 'applied', True) else p.node()
                 desc = format_desc(p.description(), self.descwidth)
                 buf += '<tr><td class="label"><b>Child:</b></td>'\
-                       '<td colspan=5><span class="rev_number">%d</span>:'\
+                       '<td colspan=5><span class="rev_number">%s</span>:'\
                        '<a href="%s" class="rev_hash">%s</a>&nbsp;'\
                        '<span class="short_desc"><i>%s</i></span></td></tr>'\
                        '\n' % (p.rev(), p.rev(), short, desc)
@@ -491,6 +496,9 @@ class RevDisplay(QtGui.QTextBrowser):
         buf += "</table>\n"
         desc = unicode(ctx.description(), 'utf-8', 'replace')
         if self.rst_action is not None  and self.rst_action.isChecked():
+            replace = cfg.getFancyReplace()
+            if replace:
+                desc = replace(desc)
             desc = rst2html(desc).decode('utf-8')
         else:
             desc = raw2html(desc)
