@@ -1,5 +1,4 @@
 # -*- coding: iso-8859-1 -*-
-#!/usr/bin/env python
 # main.py - qt4-based hg rev log browser
 #
 # Copyright (C) 2007-2010 Logilab. All rights reserved.
@@ -17,10 +16,12 @@ from PyQt4 import QtCore, QtGui, Qsci
 from mercurial import ui, hg
 from mercurial import util
 
+from hgviewlib.application import HgRepoViewer as _HgRepoViewer
 from hgviewlib.util import tounicode, has_closed_branch_support
 from hgviewlib.util import rootpath, find_repository
 from hgviewlib.hggraph import diff as revdiff
 from hgviewlib.decorators import timeit
+from hgviewlib.config import HgConfig
 
 from hgviewlib.qt4 import icon as geticon
 from hgviewlib.qt4.hgrepomodel import HgRepoListModel, HgFileListModel
@@ -41,11 +42,12 @@ connect = QtCore.QObject.connect
 SIGNAL = QtCore.SIGNAL
 
 
-class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin):
+class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin, _HgRepoViewer):
     """hg repository viewer/browser application"""
     _uifile = 'hgqv.ui'
     def __init__(self, repo, fromhead=None):
         self.repo = repo
+        self.cfg = HgConfig(repo.ui)
         self._closed_branch_supp = has_closed_branch_support(self.repo)
 
         # these are used to know where to go after a reload
@@ -157,6 +159,16 @@ class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin):
 
         self.branch_label_action = self.toolBar_treefilters.addWidget(self.branch_label)
         self.branch_comboBox_action = self.toolBar_treefilters.addWidget(self.branch_comboBox)
+        # hidden changeset action
+        self.hidden_action = QtGui.QAction(self.tr('Show Hidden'), self)
+        self.hidden_action.setCheckable(True)
+        self.hidden_action.setChecked(self.cfg.getShowHidden())
+        self.hidden_action.setToolTip(self.tr('Show hidden changeset'))
+        self.hidden_action.setStatusTip(self.tr('Show hidden changeset'))
+        self.toolBar_treefilters.addAction(self.hidden_action)
+        connect(self.hidden_action, SIGNAL('triggered()'),
+                self.refreshRevisionTable)
+        # separator
         self.toolBar_treefilters.addSeparator()
 
         self.startrev_label = QtGui.QToolButton()
@@ -171,13 +183,13 @@ class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin):
         follow_action.setStatusTip("Follow changeset history from start revision")
         self.startrev_follow_action = follow_action
         self.startrev_label.setMenu(self.startrev_menu)
+        connect(self.startrev_entry, SIGNAL('editingFinished()'),
+                self.refreshRevisionTable)
 
         self.revscompl_model = QtGui.QStringListModel(['tip'])
         self.revcompleter = QtGui.QCompleter(self.revscompl_model, self)
         self.startrev_entry.setCompleter(self.revcompleter)
 
-        connect(self.startrev_entry, SIGNAL('editingFinished()'),
-                self.refreshRevisionTable)
         connect(self.startrev_follow_action, SIGNAL('toggled(bool)'),
                 self.refreshRevisionTable)
         self.startrev_label_action = self.toolBar_treefilters.addWidget(self.startrev_label)
@@ -441,7 +453,7 @@ class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin):
         Callback called when a revision is selected in the revisions table
         """
         if self.repomodel.graph:
-            ctx = self.repomodel.repo.changectx(rev)
+            ctx = self.repomodel.repo[rev]
             self.textview_status.setContext(ctx)
             self.textview_header.displayRevision(ctx)
             self.filelistmodel.setSelectedRev(ctx)
@@ -458,11 +470,13 @@ class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin):
 
     def _getrepomtime(self):
         """Return the last modification time for the repo"""
-        watchedfiles = [(self.repo.root, ".hg", "store", "00changelog.i"),
-                        (self.repo.root, ".hg", "dirstate")]
+        watchedfiles = [(self.repo.root, ".hg", "store"),
+                        (self.repo.root, ".hg", "store", "00changelog.i"),
+                        (self.repo.root, ".hg", "dirstate"),
+                        (self.repo.root, ".hg", "store", "phasesroots"),]
         watchedfiles = [os.path.join(*wf) for wf in watchedfiles]
         mtime = [os.path.getmtime(wf) for wf in watchedfiles \
-                 if os.path.isfile(wf)]
+                 if os.path.exists(wf)]
         if mtime:
             return max(mtime)
         # humm, directory has probably been deleted, exiting...
@@ -496,6 +510,7 @@ class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin):
             return
         startrev = self.repo.changectx(startrev).rev()
         follow = self.startrev_follow_action.isChecked()
+        self.repomodel.show_hidden = self.hidden_action.isChecked()
         self.revscompl_model.setStringList(self.repo.tags().keys())
 
         self.repomodel.setRepo(self.repo, branch=branch, fromhead=startrev,
@@ -522,89 +537,3 @@ class HgRepoViewer(QtGui.QMainWindow, HgDialogMixin):
         w.raise_()
         w.activateWindow()
 
-
-def main():
-    from optparse import OptionParser
-    usage = '''%prog [options] [filename]
-
-    Starts a visual hg repository navigator.
-
-    - With no options, starts the main repository navigator.
-
-    - If a filename is given, starts in filelog diff mode (or in
-      filelog navigation mode if -n option is set).
-
-    - With -r option, starts in manifest viewer mode for given
-      revision.
-    '''
-
-    parser = OptionParser(usage)
-    parser.add_option('-R', '--repository', dest='repo',
-                      help='location of the repository to explore')
-    parser.add_option('-r', '--rev', dest='rev', default=None,
-                      help='start in manifest navigation mode at rev R')
-    parser.add_option('-n', '--navigate', dest='navigate', default=False,
-                      action="store_true",
-                      help='(with filename) start in navigation mode')
-
-    opts, args = parser.parse_args()
-
-    if opts.navigate and len(args) != 1:
-        parser.error("You must provide a filename to start in navigate mode")
-
-    if len(args) > 1:
-        parser.error("Provide at most one file name")
-
-    dir_ = None
-    if opts.repo:
-        dir_ = opts.repo
-    else:
-        dir_ = os.getcwd()
-    dir_ = find_repository(dir_)
-
-    try:
-        u = ui.ui()
-        repo = hg.repository(u, dir_)
-    except:
-        parser.error("You are not in a repo, are you?")
-
-    # make Ctrl+C works
-    import signal
-    signal.signal(signal.SIGINT, signal.SIG_DFL)
-
-    app = QtGui.QApplication(sys.argv)
-    from hgviewlib.qt4 import setup_font_substitutions
-    setup_font_substitutions()
-
-    if len(args) == 1:
-        filename = rootpath(repo, opts.rev, args[0])
-        if filename is None:
-            parser.error("%s is not a tracked file" % args[0])
-
-        # should be a filename of a file managed in the repo
-        if opts.navigate:
-            mainwindow = FileViewer(repo, filename)
-        else:
-            mainwindow = FileDiffViewer(repo, filename)
-    else:
-        rev = opts.rev
-        if rev is not None:
-            try:
-                repo.changectx(rev)
-            except RepoError, e:
-                parser.error("Cannot find revision %s" % rev)
-            else:
-                mainwindow = ManifestViewer(repo, rev)
-        else:
-            mainwindow = HgRepoViewer(repo)
-
-    mainwindow.show()
-    sys.exit(app.exec_())
-
-
-if __name__ == "__main__":
-    # remove current dir from sys.path
-    while sys.path.count('.'):
-        sys.path.remove('.')
-        print 'removed'
-    main()
