@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # util functions
 #
-# Copyright (C) 2009-2010 Logilab. All rights reserved.
+# Copyright (C) 2009-2011 Logilab. All rights reserved.
 #
 # This software may be used and distributed according to the terms
 # of the GNU General Public License, incorporated herein by reference.
@@ -10,7 +10,9 @@ Several helper functions
 """
 import os
 import string
-from mercurial import cmdutil
+from mercurial import ui
+
+from hgviewlib.hgpatches.scmutil import match
 
 def tounicode(string):
     """
@@ -81,7 +83,7 @@ def rootpath(repo, rev, path):
     path is relative to cwd
     """
     ctx = repo[rev]
-    filenames = list(ctx.walk(cmdutil.match(repo, [path], {})))
+    filenames = list(ctx.walk(match(ctx, [path], {})))
     if len(filenames) != 1 or filenames[0] not in ctx.manifest():
         return None
     else:
